@@ -60,8 +60,6 @@ extern "C" {
 
 #include <string>
 
-#include "voms_api.h"
-
 #include <iostream>
 #include <iomanip>
 
@@ -70,9 +68,6 @@ extern "C" {
 #define MS_CALLBACK
 #endif
 
-static bool check_cert(X509 *cert, verror_type &error, std::string &cdir);
-static int MS_CALLBACK cb(int ok, X509_STORE_CTX *ctx);
-static bool id_data(std::string &subj, std::string &ca, verror_type &error);
 extern bool contact(const std::string &hostname, int port, const std::string &contact,
                     const std::string &command, std::string &buffer, std::string &subject, std::string &ca, verror_type &error);
 extern bool retrieve(X509 *cert, STACK_OF(X509) *chain, recurse_type how, 
