@@ -50,6 +50,7 @@ EXTERN_C_BEGIN
 /**********************************************************************
                              Include header files
 **********************************************************************/
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -57,6 +58,7 @@ EXTERN_C_BEGIN
 #include "openssl/crypto.h"
 
 
+
 #if SSLEAY_VERSION_NUMBER < 0x0090581fL
 #define RAND_add(a,b,c) RAND_seed(a,b)
 #define RAND_status() 1
@@ -293,37 +295,11 @@ ERR_set_continue_needed(void);
 #define PRXYERR_R_UNKNOWN_CRIT_EXT             PRXYERR_R_BASE + 63
 /* NOTE: Don't go over 1500 here or will conflict with errors in scutils.h */
 
-/* constants for gsi error messages 
- *  this information is kept internally by the
- *  proxy_cred_desc structure
- */
-
-#define CRED_TYPE_PERMANENT             0
-#define CRED_TYPE_PROXY                 1
-#define CRED_OWNER_SERVER               0  
-#define CRED_OWNER_USER                 1 
 
 /**********************************************************************
                                Type definitions
 **********************************************************************/
 
-typedef struct proxy_cred_desc_struct
-{
-    X509 *                              ucert;
-    EVP_PKEY *                          upkey;
-    STACK_OF(X509) *                    cert_chain;
-    SSL_CTX *                           gs_ctx;
-    /* smart card session handle */
-    unsigned long                       hSession;
-    /* private key session handle */
-    unsigned long                       hPrivKey; 
-    char *                              certdir;
-    char *                              certfile;
-    int                                 num_null_enc_ciphers;
-    int                      	        type;  /* for gsi error messages */
-    int                                 owner; /* for gsi error messages */
-} proxy_cred_desc;
-
 /* proxy_verify_ctx_desc - common to all verifys */
 
 typedef struct proxy_verify_ctx_desc_struct {
@@ -361,12 +337,6 @@ struct proxy_verify_desc_struct {
 int
 ERR_load_prxyerr_strings(int i);
 
-proxy_cred_desc * 
-proxy_cred_desc_new();
-
-int
-proxy_cred_desc_free(proxy_cred_desc * pcd);
-
 int proxy_load_user_cert_and_key_pkcs12(const char *user_cert,
                                         X509 **cert,
                                         STACK_OF(X509) **stack,
@@ -387,36 +357,16 @@ proxy_load_user_cert(
     const char *                        user_cert,
     X509 **                             certificate,
     int                                 (*pw_cb)(),
-    BIO *                               bp,
     unsigned long *                     hSession);
+
 int
 proxy_load_user_key(
     EVP_PKEY **                         private_key,
     X509 * ucert,
     const char *                        user_key,
     int                                 (*pw_cb)(),
-    BIO *                               bp,
     unsigned long *                     hSession);
 
-int
-proxy_create_local(
-    proxy_cred_desc *                   pcd,
-    const char *                        outfile,
-    int                                 hours,
-    int                                 bits,
-    int                                 limited_proxy,
-    int                                 proxyver,
-    int                                 (*kpcallback)(),
-    char *                              buffer,
-    int                                 length);
-
-
-int
-proxy_init_cred(
-    proxy_cred_desc *                   pcd,
-    int                                 (*pw_cb)(),
-    BIO *                               bp);
-
 void
 proxy_verify_init(
     proxy_verify_desc *                 pvd,
@@ -472,22 +422,28 @@ proxy_sign(
     STACK_OF(X509_EXTENSION) *          extensions,
     int                                 limited_proxy,
     int                                 proxyver,
-    const char *                        newdn
+    const char *                        newdn,
+    const char *                        newissuer,
+    int                                 pastproxy,
+    const char *                        newserial,
+    int                                 selfsigned
 );
 
 int
 proxy_sign_ext(
     X509 *                              user_cert,
     EVP_PKEY *                          user_private_key,
-    const EVP_MD *                            method,
+    const EVP_MD *                      method,
     X509_REQ *                          req,
     X509 **                             new_cert,
     X509_NAME *                         subject_name,
     X509_NAME *                         issuer_name,    
     int                                 seconds,
-    int                                 serial_num,
     STACK_OF(X509_EXTENSION) *          extensions,
-    int proxyver);
+    int                                 proxyver,
+    int                                 pastproxy,
+    const char *                        newserial,
+    int                                 selfsigned);
 
 int
 proxy_check_subject_name(
@@ -520,18 +476,12 @@ proxy_marshal_bp(
 int
 proxy_load_user_proxy(
     STACK_OF(X509) *                    cert_chain,
-    const char *                        file,
-    BIO *                               bp);
+    const char *                        file);
 
 int
 proxy_get_base_name(
     X509_NAME *                         subject);
 
-int proxy_password_callback_no_prompt(
-    char *,
-    int,
-    int);
-
 X509_EXTENSION *
 proxy_extension_class_add_create(
     void *                              buffer, 
@@ -545,35 +495,23 @@ time_t
 ASN1_UTCTIME_mktime(
     ASN1_UTCTIME *                     ctm);
 
-#if 0
-/*
- * Functions similar to {i2d,d2i}_X509_bio which write/read a integer
- * (ASN.1 representation) to/from a bio
- */
-
-int
-i2d_integer_bio(
-    BIO *                               bp,
-    long                                v);
-
-long
-d2i_integer_bio(
-    BIO *                               bp,
-    long *                              v);
-#endif
+time_t ASN1_TIME_mktime(ASN1_TIME *ctm);
 
 int PRIVATE determine_filenames(char **cacert, char **certdir, char **outfile,
                                 char **certfile, char **keyfile, int noregen);
-int PRIVATE load_credentials(const char *certname, const char *keyname,
+int load_credentials(const char *certname, const char *keyname,
                              X509 **cert, STACK_OF(X509) **stack, EVP_PKEY **key,
                              int (*callback)());
 int PRIVATE load_certificate_from_file(FILE *file, X509 **cert, 
                                        STACK_OF(X509) **stack);
 
-int PRIVATE
+int
 proxy_app_verify_callback(X509_STORE_CTX *ctx, UNUSED(void *empty));
 
+STACK_OF(X509) *load_chain(BIO *in, char*);
+
+int my_txt2nid(char *name);
+
 EXTERN_C_END
 
-int my_txt2nid(char *name);
 #endif /* _SSLUTILS_H */
