@@ -22,10 +22,6 @@
  * follows.
  *
  *********************************************************************/
-/*
- * No original header was present, but the still_valid() function was
- * adapted from original Globus code.
- */
 
 /**********************************************************************
                              Include header files
@@ -33,15 +29,12 @@
 #include "config.h"
 #include "replace.h"
 
-//const std::string VERSION         = "0.1";
-
 extern "C" {
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-  //#include <getopt.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/mman.h>
@@ -55,9 +48,13 @@ extern "C" {
 #include "openssl/asn1.h"
 #include "openssl/evp.h"
 #include "openssl/x509.h"
+#include "openssl/x509v3.h"
 #include "openssl/pem.h"
 #include "openssl/ssl.h"
 #include "openssl/rsa.h"
+#include "openssl/conf.h"
+#include "openssl/stack.h"
+#include "openssl/opensslv.h"
 
 #ifdef USE_PKCS11
 #include "scutils.h"
@@ -95,9 +92,10 @@ const std::string SUBPACKAGE      = "voms-proxy-info";
 static bool test_proxy();
 
 static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd);
-static STACK_OF(X509) *load_chain(char *certfile);
+static STACK_OF(X509) *load_chain_from_file(char *certfile);
 static time_t stillvalid(ASN1_TIME *ctm);
-static char *proxy_type(X509 *cert);
+static const char *proxy_type(X509 *cert);
+static std::string getKeyUsage(X509 *cert);
 
 std::string program;
 
@@ -141,6 +139,7 @@ static bool        dont_verify_ac = false;
 static bool        targets = false;
 static bool        included = false;
 static bool        printuri = false;
+static bool        keyusage = false;
 
 int
 main(int argc, char **argv)
@@ -153,7 +152,7 @@ main(int argc, char **argv)
   else
     program = argv[0];
 
-  static char *LONG_USAGE = 
+  static std::string LONG_USAGE = 
     "\n\n"
     "Syntax: voms-proxy-info [-help][-file proxyfile][-subject][...][-exists [-hours H][-bits B][-valid H:M]]\n\n"
     "   Options\n"
@@ -186,6 +185,7 @@ main(int argc, char **argv)
     "      -actimeleft           Time (in seconds) until AC expires\n"
     "      -serial               AC serial number \n"
     "      -uri                  Server URI\n"
+    "      -keyusage             Print content of KeyUsage extension.\n"
     "\n"
     "   [options to -exists]      (if none are given, H = B = 0 are assumed)\n"
     "      -valid H:M            time requirement for proxy to be valid\n"
@@ -230,6 +230,7 @@ main(int argc, char **argv)
     {"targets",        0, (int *)&targets,        OPT_BOOL},
     {"included-file",  0, (int *)&included,       OPT_BOOL},
     {"uri",            0, (int *)&printuri,       OPT_BOOL},
+    {"keyusage",       0, (int *)&keyusage,       OPT_BOOL},
     {0, 0, 0, 0}
   };
 
@@ -298,36 +299,35 @@ int numbits(X509 *cert)
   return bits;
 }
 
-static char *proxy_type(X509 *cert)
+static const char *findlast(const char *haystack, const char *needle)
+{
+  const char *point = strstr(haystack, needle);
+  const char *tmp = point;
+
+  while (tmp) {
+    tmp = strstr(tmp+1, needle);
+    if (tmp)
+      point = (char*)tmp;
+  }
+
+  return point;
+}
+
+static const char *proxy_type(X509 *cert)
 {
   char *buffer = X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0);
-  char *point1 = strstr(buffer,"CN=proxy");
-
-  char *tmp = point1;
-  while (tmp) {
-    tmp = strstr(tmp +1, "CN=proxy");
-    if (tmp)
-      point1 = tmp;
-  }
-  char *point2 = strstr(buffer,"CN=limited proxy");
-
-  tmp = point2;
-  while (tmp) {
-    tmp = strstr(tmp +1, "CN=limited proxy");
-    if (tmp)
-      point2 = tmp;
-  }
-
-  int len = strlen(buffer);
+  const char *point1 = findlast(buffer,"CN=proxy");
+  const char *point2 = findlast(buffer,"CN=limited proxy");
 
   OPENSSL_free(buffer);
 
-  if (point1)
-    if (len == ((point1 - buffer) + 8))
+  /*
+   * check whether "proxy" or "limited proxy" came last
+   */
+  if (point1 > point2)
       return "proxy";
 
-  if (point2)
-    if (len == ((point2 - buffer) + 16))
+  if (point2 > point1)
       return "limited proxy";
 
   int nidv3 = OBJ_txt2nid(PROXYCERTINFO_V3);
@@ -354,13 +354,10 @@ static bool
 test_proxy()
 {
   char *ccaf;
-  char *cd;
   char *of;
-  char *cf;
-  char *kf;
+  char *inof;
   bool res = false;
   BIO *bio_err;
-  proxy_cred_desc *pcd;
   BIO  *in = NULL;
   X509 *x  = NULL;
   STACK_OF(X509) *chain = NULL;
@@ -377,69 +374,23 @@ test_proxy()
   if ((bio_err=BIO_new(BIO_s_file())) != NULL)
     BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
 
-  if ((pcd = proxy_cred_desc_new()) == NULL)
-    goto err;
-
-  pcd->type = CRED_TYPE_PERMANENT;
-
   /*
    * These 5 const_cast are allowed because proxy_get_filenames will
    * overwrite the pointers, not the data itself.
    */
   ccaf = NULL;
-  cd   = NULL;
-  of   = (file.empty() ? NULL : const_cast<char *>(file.c_str()));
-  cf   = NULL;
-  kf   = NULL;
+  inof = of   = (file.empty() ? NULL : const_cast<char *>(file.c_str()));
     
-  if (!determine_filenames(&ccaf, &cd, &of, &cf, &kf, 0)) {
-    /* dump error from proxy routines. */
-  unsigned long l;
-  char buf[256];
-#if SSLEAY_VERSION_NUMBER  >= 0x00904100L
-  const char *file;
-#else
-  char *file;
-#endif
-  char *dat;
-  int line;
-    
-  /* WIN32 does not have the ERR_get_error_line_data */ 
-  /* exported, so simulate it till it is fixed */
-  /* in SSLeay-0.9.0 */
-  
-  while ( ERR_peek_error() != 0 ) {
-    
-    int i;
-    ERR_STATE *es;
-      
-    es = ERR_get_state();
-    i = (es->bottom+1)%ERR_NUM_ERRORS;
-    
-    if (es->err_data[i] == NULL)
-      dat = strdup("");
-    else
-      dat = strdup(es->err_data[i]);
-
-    if (dat) {
-      l = ERR_get_error_line(&file, &line);
-
-      if (debug)
-        std::cerr << ERR_error_string(l,buf) << ":"
-                  << file << ":" << line << dat << std::endl;
-      else
-        std::cerr << ERR_reason_error_string(l) << dat
-                  << "\nFunction: " << ERR_func_error_string(l) << std::endl;
-    }
-    
-    free(dat);
-  }
+  if (!determine_filenames(&ccaf, NULL, &of, NULL, NULL, 0)) {
+    std::string output = OpenSSLError(debug);
 
+    std::cerr << output;
 
     goto err;
   }
 
-  file = std::string(of);
+  if (of != inof)
+    file = std::string(of);
 
   in = BIO_new(BIO_s_file());
   if (in) {
@@ -449,7 +400,9 @@ test_proxy()
         std::cerr << "Couldn't find a valid proxy." << std::endl;
         goto err;
       }
-      chain = load_chain(of);
+      chain = load_chain_from_file(of);
+
+
       vomsdata d("","");
       if (!dont_verify_ac) {
           d.SetVerificationType((verify_type)(VERIFY_SIGN | VERIFY_KEY));
@@ -485,57 +438,30 @@ test_proxy()
   BIO_free(in);
   BIO_free(bio_err);
 
+  X509_free(x);
+  sk_X509_pop_free(chain, X509_free);
+  if (of != inof)
+    free(of);
+
   return res;
 }
 
-static STACK_OF(X509) *load_chain(char *certfile)
+static STACK_OF(X509) *load_chain_from_file(char *certfile)
 {
-  STACK_OF(X509_INFO) *sk=NULL;
-  STACK_OF(X509) *stack=NULL, *ret=NULL;
-  BIO *in=NULL;
-  X509_INFO *xi;
-  int first = 1;
+  BIO *bio = NULL;
+  STACK_OF(X509) *stack = NULL;
 
-  if(!(stack = sk_X509_new_null())) {
-    printf("memory allocation failure\n");
-    goto end;
+  bio = BIO_new_file(certfile, "r");
+
+  if (bio) {
+    stack = load_chain(bio, certfile);
+    BIO_free(bio);
   }
-
-  if(!(in=BIO_new_file(certfile, "r"))) {
+  else {
     printf("error opening the file, %s\n",certfile);
-    goto end;
   }
 
-  /* This loads from a file, a stack of x509/crl/pkey sets */
-  if(!(sk=PEM_X509_INFO_read_bio(in,NULL,NULL,NULL))) {
-    printf("error reading the file, %s\n",certfile);
-    goto end;
-  }
-
-  /* scan over it and pull out the certs */
-  while (sk_X509_INFO_num(sk)) {
-    /* skip first cert */
-    if (first) {
-      first = 0;
-      continue;
-    }
-    xi=sk_X509_INFO_shift(sk);
-    if (xi->x509 != NULL) {
-      sk_X509_push(stack,xi->x509);
-      xi->x509=NULL;
-    }
-    X509_INFO_free(xi);
-  }
-  if(!sk_X509_num(stack)) {
-    printf("no certificates in file, %s\n",certfile);
-    sk_X509_free(stack);
-    goto end;
-  }
-  ret=stack;
-end:
-  BIO_free(in);
-  sk_X509_INFO_free(sk);
-  return(ret);
+  return stack;
 }
 
 
@@ -544,7 +470,7 @@ convtime(std::string data)
 {
   ASN1_TIME *t= ASN1_TIME_new();
 
-  t->data   = (unsigned char *)(data.data());
+  t->data   = (unsigned char*)strdup(data.data());
   t->length = data.size();
   switch(t->length) {
   case 10:
@@ -580,7 +506,6 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
       std::cout << "=== Proxy Chain Information ===" << std::endl;
 
       for (start = sk_X509_num(chain)-1; start >= 1; start--) {
-        char *type = NULL;
         int totbits = 0;
         time_t leftcert = 0;
 
@@ -588,14 +513,22 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
         totbits = numbits(cert);
         leftcert = stillvalid(X509_get_notAfter(cert)) - now;
         leftcert = (leftcert < 0) ? 0 : leftcert;
-        type = proxy_type(cert);
 
-        std::cout << "subject   : " << X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0) << "\n";
-        std::cout << "issuer    : " << X509_NAME_oneline(X509_get_issuer_name(cert), NULL, 0) << "\n";
-        
+        const char *type = proxy_type(cert);
+
+        char *name = X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0);
+        std::cout << "subject   : " << name << "\n";
+        OPENSSL_free(name);
+
+        name = X509_NAME_oneline(X509_get_issuer_name(cert), NULL, 0);
+        std::cout << "issuer    : " << name << "\n";
+        OPENSSL_free(name);
+
         if (strcmp(type, "unknown") != 0)
-          std::cout << "type      : " << proxy_type(cert) << "\n";
-        
+          std::cout << "type      : " << type << "\n";
+
+        if (all || keyusage)
+          std::cout << "key usage : " << getKeyUsage(cert) << "\n";
         std::cout << "strength  : " << totbits << " bits" << "\n";
         std::cout << "timeleft  : " << leftcert/3600 << ":" << std::setw(2) << std::setfill('0') 
                   << (leftcert%3600)/60 << ":" << std::setw(2) << std::setfill('0') << (leftcert%3600)%60 << "\n\n";
@@ -605,9 +538,15 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
   }
  
   if (defaultinfo || all || text) {
-    std::cout << "subject   : " << X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0) << "\n";
-    std::cout << "issuer    : " << X509_NAME_oneline(X509_get_issuer_name(cert), NULL, 0) << "\n";
-    std::cout << "identity  : " << X509_NAME_oneline(X509_get_issuer_name(cert), NULL, 0) << "\n";
+    char * name = X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0);
+    std::cout << "subject   : " << name << "\n";
+    OPENSSL_free(name);
+
+    name = X509_NAME_oneline(X509_get_issuer_name(cert), NULL, 0);
+    std::cout << "issuer    : " << name  << "\n";
+    std::cout << "identity  : " << name << "\n";
+    OPENSSL_free(name);
+
     std::cout << "type      : " << proxy_type(cert) << "\n";
     std::cout << "strength  : " << totbits << " bits" << "\n";
     std::cout << "path      : " << file << "\n";
@@ -615,33 +554,44 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
               << (leftcert%3600)/60 << ":" << std::setw(2) << std::setfill('0') << (leftcert%3600)%60 << "\n";
     if (!vd.extra_data.empty())
       std::cout << "included  : "  << vd.extra_data << "\n";
-
+    if (all || text || keyusage)
+      std::cout << "key usage : " << getKeyUsage(cert) << "\n";
   }    
   
   if (all) {
-    for (std::vector<voms>::iterator v = vd.data.begin(); v != vd.data.end(); v++) {
+    std::vector<voms>::const_iterator vend = vd.data.end();
+    for (std::vector<voms>::iterator v = vd.data.begin(); v != vend; ++v) {
       ASN1_TIME * after  = convtime(v->date2);
       leftac = stillvalid(after) - now;	
       leftac = (leftac<0) ? 0 : leftac;
+      ASN1_TIME_free(after);
 
       std::cout << "=== VO " << v->voname << " extension information ===\n";
       std::cout << "VO        : " << v->voname << "\n";
       std::cout << "subject   : " << v->user << "\n";
       std::cout << "issuer    : " << v->server << "\n";
 
-      for (std::vector<std::string>::iterator s = v->fqan.begin(); s != v->fqan.end(); s++)
+      std::vector<std::string>::const_iterator send = v->fqan.end();
+      for (std::vector<std::string>::const_iterator s = v->fqan.begin(); s != send; ++s)
         std::cout << "attribute : " << *s << "\n";
+
       std::vector<attributelist> alist = v->GetAttributes();
-      for (std::vector<attributelist>::iterator s = alist.begin(); s != alist.end(); s++)
-        for (std::vector<attribute>::iterator t = s->attributes.begin(); t != s->attributes.end(); t++)
+      std::vector<attributelist>::const_iterator aend = alist.end();
+      
+      for (std::vector<attributelist>::const_iterator s = alist.begin(); s != aend; ++s) {
+        std::vector<attribute>::const_iterator tend = s->attributes.end();
+        for (std::vector<attribute>::const_iterator t = s->attributes.begin(); t != tend; ++t)
           std::cout << "attribute : " << t->name + " = " + t->value + 
             (t->qualifier.empty() ? "" : " (" + t->qualifier + ")") << std::endl;
+      }
+
       std::cout << "timeleft  : " << leftac/3600 << ":" << std::setw(2) << std::setfill('0')
                 << (leftac%3600)/60 << ":" << std::setw(2) << std::setfill('0') << (leftac%3600)%60 << "\n";
 
       std::vector<std::string> targetlist = v->GetTargets();
       if (!targetlist.empty()) {
-        for (std::vector<std::string>::iterator targ = targetlist.begin(); targ != targetlist.end(); targ++)
+        std::vector<std::string>::const_iterator tend = targetlist.end();
+        for (std::vector<std::string>::const_iterator targ = targetlist.begin(); targ != tend; ++targ)
           std::cout << "target    : " << *targ << "\n";
       }
       std::cout << "uri       : " << v->uri  << "\n";
@@ -649,12 +599,22 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
     }
   }
 
-  if (subject)
-    std::cout << X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0) << "\n";
-  if (issuer)
-    std::cout << X509_NAME_oneline(X509_get_issuer_name(cert), NULL, 0) << "\n";
-  if (identity)
-    std::cout << X509_NAME_oneline(X509_get_issuer_name(cert), NULL, 0) << "\n";
+  if (subject) {
+    char *name = X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0);
+    std::cout << name << "\n";
+    OPENSSL_free(name);
+  }
+  if (issuer) {
+    char *name = X509_NAME_oneline(X509_get_issuer_name(cert), NULL, 0);
+    std::cout << name << "\n";
+    OPENSSL_free(name);
+  }
+  if (identity) {
+    char *name = X509_NAME_oneline(X509_get_issuer_name(cert), NULL, 0);
+    std::cout << name << "\n";
+    OPENSSL_free(name);
+  }
+
   if (type)
     std::cout << proxy_type(cert) << "\n";
   if (strength)
@@ -671,12 +631,14 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
   if(text) {
     X509 *tmp = X509_dup(cert);
     X509_print_fp(stdout, tmp);
+    X509_free(tmp);
 
     if (dochain) {
       for (int start = sk_X509_num(chain)-1; start >= 1; start--) {
         X509 *tmp = sk_X509_value(chain, start);
         X509 *cert = X509_dup(tmp);
         X509_print_fp(stdout, cert);
+        X509_free(cert);
       }
     }
   }
@@ -685,7 +647,8 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
     if (vo || acsubject || acissuer || actimeleft || fqan || serial || targets || printuri)
       res = false;
 
-  for (std::vector<voms>::iterator v = vd.data.begin(); v != vd.data.end(); v++) {
+  std::vector<voms>::const_iterator vend = vd.data.end();
+  for (std::vector<voms>::iterator v = vd.data.begin(); v != vend; ++v) {
     if(vo)
         std::cout << v->voname << "\n";
 
@@ -701,12 +664,14 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
     ASN1_TIME * after  = convtime(v->date2);
     leftac = stillvalid(after) - now;
     leftac = (leftac<0) ? 0 : leftac;
+    ASN1_TIME_free(after);
 
     if (actimeleft)
         std::cout << leftac << "\n";
 
     if (fqan) {
-      for (std::vector<std::string>::iterator s = v->fqan.begin(); s != v->fqan.end(); s++)
+      std::vector<std::string>::const_iterator send = v->fqan.end();
+      for (std::vector<std::string>::const_iterator s = v->fqan.begin(); s != send; ++s)
         std::cout << *s << "\n";
       if (v->fqan.empty())
         res = false;
@@ -719,7 +684,8 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
       std::vector<std::string> targetlist = v->GetTargets();
 
       if (!targetlist.empty()) {
-        for (std::vector<std::string>::iterator targ = targetlist.begin(); targ != targetlist.end(); targ++)
+        std::vector<std::string>::const_iterator tend = targetlist.end();
+        for (std::vector<std::string>::const_iterator targ = targetlist.begin(); targ != tend; ++targ)
           std::cout << "target    : " << *targ << "\n";
       }
     }
@@ -739,10 +705,13 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
   /* -acexists */
 
   if(res) {
-    for (std::vector<std::string>::iterator i = acexists.begin(); i != acexists.end(); ++i) {
+    std::vector<std::string>::const_iterator acend = acexists.end();
+
+    for (std::vector<std::string>::const_iterator i = acexists.begin(); i != acend; ++i) {
       bool found = false;
 
       if(res) {
+        std::vector<voms>::const_iterator vend = vd.data.end();
         for (std::vector<voms>::iterator v = vd.data.begin(); v != vd.data.end(); ++v) {
           if(v->voname == *i) {
             found = true;
@@ -761,104 +730,38 @@ static bool print(X509 *cert, STACK_OF(X509) *chain, vomsdata &vd)
 
 static time_t stillvalid(ASN1_TIME *ctm)
 {
-  char     *str;
-  time_t    offset;
-  time_t    newtime;
-  char      buff1[32];
-  char     *p;
-  int       i;
-  struct tm tm;
-  int       size;
+  return ASN1_TIME_mktime(ctm);
+}
 
-  switch (ctm->type) {
-  case V_ASN1_UTCTIME:
-    size=10;
-    break;
-  case V_ASN1_GENERALIZEDTIME:
-    size=12;
-    break;
-  }
-  p = buff1;
-  i = ctm->length;
-  str = (char *)ctm->data;
-  if ((i < 11) || (i > 17)) {
-    newtime = 0;
-  }
-  memcpy(p,str,size);
-  p += size;
-  str += size;
+static std::string getKeyUsage(X509 *cert)
+{
+  STACK_OF(CONF_VALUE) *confs = NULL;
+  ASN1_BIT_STRING *usage = NULL;
 
-  if ((*str == 'Z') || (*str == '-') || (*str == '+')) {
-    *(p++)='0'; *(p++)='0';
-  }
-  else {
-    *(p++)= *(str++); *(p++)= *(str++);
-  }
-  *(p++)='Z';
-  *(p++)='\0';
+  std::string keyusage;
 
-  if (*str == 'Z') {
-    offset=0;
-  }
-  else {
-    if ((*str != '+') && (str[5] != '-')) {
-      newtime = 0;
-    }
-    offset=((str[1]-'0')*10+(str[2]-'0'))*60;
-    offset+=(str[3]-'0')*10+(str[4]-'0');
-    if (*str == '-') {
-      offset=-offset;
-    }
-  }
-
-  tm.tm_isdst = 0;
-  int index = 0;
-  if (ctm->type == V_ASN1_UTCTIME) {
-    tm.tm_year  = (buff1[index++]-'0')*10;
-    tm.tm_year += (buff1[index++]-'0');
-  }
-  else {
-    tm.tm_year  = (buff1[index++]-'0')*1000;
-    tm.tm_year += (buff1[index++]-'0')*100;
-    tm.tm_year += (buff1[index++]-'0')*10;
-    tm.tm_year += (buff1[index++]-'0');
-  }
-
-  if (tm.tm_year < 70) {
-    tm.tm_year+=100;
-  }
-
-  if (tm.tm_year > 1900) {
-    tm.tm_year -= 1900;
-  }
-
-  tm.tm_mon   = (buff1[index++]-'0')*10;
-  tm.tm_mon  += (buff1[index++]-'0')-1;
-  tm.tm_mday  = (buff1[index++]-'0')*10;
-  tm.tm_mday += (buff1[index++]-'0');
-  tm.tm_hour  = (buff1[index++]-'0')*10;
-  tm.tm_hour += (buff1[index++]-'0');
-  tm.tm_min   = (buff1[index++]-'0')*10;
-  tm.tm_min  += (buff1[index++]-'0');
-  tm.tm_sec   = (buff1[index++]-'0')*10;
-  tm.tm_sec  += (buff1[index++]-'0');
-
-  /*
-   * mktime assumes local time, so subtract off
-   * timezone, which is seconds off of GMT. first
-   * we need to initialize it with tzset() however.
-   */
-
-  tzset();
-#if defined(HAVE_TIMEGM)
-  newtime = (timegm(&tm) + offset*60*60);
-#elif defined(HAVE_TIME_T_TIMEZONE)
-  newtime = (mktime(&tm) + offset*60*60 - timezone);
-#elif defined(HAVE_TIME_T__TIMEZONE)
-  newtime = (mktime(&tm) + offset*60*60 - _timezone);
+#if OPENSSL_VERSION_NUMBER >= 0x10000000L
+  const X509V3_EXT_METHOD *method = X509V3_EXT_get_nid(NID_key_usage);
 #else
-  newtime = (mktime(&tm) + offset*60*60);
+  X509V3_EXT_METHOD *method = X509V3_EXT_get_nid(NID_key_usage);
 #endif
 
-  return newtime;
+  if (method) {
+    confs = NULL;
+    usage = (ASN1_BIT_STRING*)X509_get_ext_d2i(cert, NID_key_usage, NULL, NULL);
+    confs = method->i2v(method, usage, confs);
+
+    for (int i =0; i < sk_CONF_VALUE_num(confs); i ++) {
+      CONF_VALUE *conf = (CONF_VALUE*)sk_CONF_VALUE_value(confs, i);
+      keyusage += std::string(conf->name);
+      if (i != (sk_CONF_VALUE_num(confs) -1))
+        keyusage += ", ";
+    }
+
+    ASN1_BIT_STRING_free(usage);
+
+    sk_CONF_VALUE_pop_free(confs, X509V3_conf_free);
+  }
+
+  return keyusage;
 }
